﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Nop.Core;
using Nop.Core.Domain.Directory;
using Nop.Core.Domain.Localization;
using Nop.Core.Domain.Orders;
using Nop.Core.Domain.Payments;
using Nop.Plugin.Payments.OmniKassa.Domain;
using Nop.Services.Configuration;
using Nop.Services.Directory;
using Nop.Services.Localization;
using Nop.Services.Logging;
using Nop.Services.Orders;
using Nop.Services.Payments;
using Nop.Services.Stores;
using Nop.Web.Framework.Controllers;
using Nop.Plugin.Payments.OmniKassa.Models;
using Nop.Plugin.Payments.OmniKassa.Validators;
using Nop.Web.Framework.Localization;

namespace Nop.Plugin.Payments.OmniKassa.Controllers
{
    public class PaymentOmniKassaController : BaseNopPaymentController
    {        
        private readonly ILocalizationService _localizationService;
        private readonly ILanguageService _languageService; 
        private readonly ISettingService _settingService;
        private readonly IPaymentService _paymentService;
        private readonly PaymentSettings _paymentSettings;
//        private readonly OmniKassaPaymentSettings _omniKassaPaymentSettings;
        private readonly IStoreService _storeService;
        private readonly IOrderProcessingService _orderProcessingService;
        private readonly IOrderService _orderService;
        private readonly IWorkContext _workContext;
        private readonly ICurrencyService _currencyService;
        private readonly CurrencySettings _currencySettings;
        private readonly HttpContextBase _httpContext;
        private readonly IStoreContext _storeContext;
        private readonly ILogger _logger;
        

        private const String PaymentSystemMethodName = "Payments.OmniKassa";


        public PaymentOmniKassaController(  ILocalizationService localizationService,
                                            ILanguageService languageService,
                                            ISettingService settingService,
                                            IPaymentService paymentService,
                                            PaymentSettings paymentSettings,
                                            //OmniKassaPaymentSettings omniKassaPaymentSettings,
                                            IStoreService storeService,
                                            IOrderProcessingService orderProcessingService,
                                            IOrderService orderService,
                                            IWorkContext workContext,
                                            ICurrencyService currencyService,
                                            CurrencySettings currencySettings,
                                            HttpContextBase httpContext,
                                            IStoreContext storeContext,
                                            ILogger logger)
        {
            _localizationService = localizationService;
            _languageService = languageService;
            _settingService = settingService;
            _paymentService = paymentService;
            _paymentSettings = paymentSettings;
  //          _omniKassaPaymentSettings = omniKassaPaymentSettings;
            _storeService = storeService;
            _orderProcessingService = orderProcessingService;
            _orderService = orderService;
            _workContext = workContext;
            _currencyService = currencyService;
            _currencySettings = currencySettings;
            _httpContext = httpContext;
            _storeContext = storeContext;
            _logger = logger;            
        }

        public ActionResult EditPopup()
        {
            var model = PrepareConfigurationModel();

            return View("Nop.Plugin.Payments.OmniKassa.Views.PaymentOmniKassa.EditPopup", model);
        }

        [HttpPost]
        public ActionResult EditPopup(ConfigurationModel model)
        {
            _settingService.SetSetting("omnikassapaymentsettings.redirectiontip", model.RedirectionTip);

            foreach (var localized in model.Locales)
            {
                SaveLocalizedResource(_localizationService, localized.LanguageId, localized.RedirectionTip, "Plugins.Payments.OmniKassa.RedirectionTip.Localized");
                SaveLocalizedResource(_localizationService, localized.LanguageId, localized.Ideal, "Plugins.Payments.OmniKassa.IdealDisplayName.Localized");
                SaveLocalizedResource(_localizationService, localized.LanguageId, localized.MasterCard, "Plugins.Payments.OmniKassa.MasterCardDisplayName.Localized");
                SaveLocalizedResource(_localizationService, localized.LanguageId, localized.Visa, "Plugins.Payments.OmniKassa.VisaDisplayName.Localized");
                SaveLocalizedResource(_localizationService, localized.LanguageId, localized.Maestro, "Plugins.Payments.OmniKassa.MaestroDisplayName.Localized");
                SaveLocalizedResource(_localizationService, localized.LanguageId, localized.MiniTix, "Plugins.Payments.OmniKassa.MiniTixDisplayName.Localized");
                SaveLocalizedResource(_localizationService, localized.LanguageId, localized.Rembours, "Plugins.Payments.OmniKassa.RemboursDisplayName.Localized");
                SaveLocalizedResource(_localizationService, localized.LanguageId, localized.Incasso, "Plugins.Payments.OmniKassa.IncassoDisplayName.Localized");
                SaveLocalizedResource(_localizationService, localized.LanguageId, localized.Acceptgiro, "Plugins.Payments.OmniKassa.AcceptgiroDisplayName.Localized");
            }

            ViewBag.AutoClose = true;

            return View("Nop.Plugin.Payments.OmniKassa.Views.PaymentOmniKassa.EditPopup", model);
        }

        [AdminAuthorize]
        [ChildActionOnly]
        public ActionResult Configure()
        {
            var model = PrepareConfigurationModel();

            return View("Nop.Plugin.Payments.OmniKassa.Views.PaymentOmniKassa.Configure", model);
        }

        [HttpPost]
        [AdminAuthorize]
        [ChildActionOnly]
        [ValidateInput(false)]
        public ActionResult Configure(ConfigurationModel model)
        {
            if (!ModelState.IsValid)
                return Configure();

            var storeScope = this.GetActiveStoreScopeConfiguration(_storeService, _workContext);
            var omniKassaSettings = _settingService.LoadSetting<OmniKassaPaymentSettings>(storeScope);
            //save settings
            omniKassaSettings.MerchantId = model.MerchantId;
            omniKassaSettings.NormalReturnUrl = model.NormalReturnUrl;
            omniKassaSettings.OmniKassaUrl = model.OmniKassaUrl;
            omniKassaSettings.AutomaticResponseUrl = model.AutomaticResponseUrl;
            omniKassaSettings.TestOmniKassaUrl = model.TestOmniKassaUrl;
            omniKassaSettings.InterfaceVersion = model.InterfaceVersion;
            omniKassaSettings.MinimumAmount = model.MinimumAmount;
            omniKassaSettings.AdditionalFee = model.AdditionalFee;
            omniKassaSettings.AdditionalFeePercentage = model.AdditionalFeePercentage;
            omniKassaSettings.SecretKey = model.SecretKey;
            omniKassaSettings.SecretKeyVersion = model.SecretKeyVersion;
            omniKassaSettings.TestMode = model.TestMode;
            omniKassaSettings.IsPaymentMethodUserChoice = model.IsPaymentMethodUserChoice;

            List<OmniKassaService> oks =
                omniKassaSettings.OmniKassaServicesList.XmlDeserializeFromString<List<OmniKassaService>>();

            foreach (var pb in model.AvailablePaymentMeanBrandList.PaymentMeanBrands)
            {
                OmniKassaService ok = oks.First(o => o.Name == pb.Name);
                ok.Active = pb.Active;
                ok.AdditionalFee = ok.AdditionalFee;
                ok.AdditionalFeePercentage = ok.AdditionalFeePercentage;
            }

            omniKassaSettings.OmniKassaServicesList = oks.XmlSerializeToString();

            //save settings
            _settingService.SaveSetting(omniKassaSettings);

            //return View("Nop.Plugin.Payments.OmniKassa.Views.PaymentOmniKassa.Configure", model);
            return Configure();
        }


        [NonAction]
        private ConfigurationModel PrepareConfigurationModel(int? storeid = null)
        {
            int storeScope;
            if (!storeid.HasValue)
            {
                storeScope = this.GetActiveStoreScopeConfiguration(_storeService, _workContext);
            }
            else
            {
                storeScope = storeid.Value;
            }

            var omniKassaSettings = _settingService.LoadSetting<OmniKassaPaymentSettings>(storeScope);

            var model = new ConfigurationModel()
                            {
                                MerchantId = omniKassaSettings.MerchantId,
                                NormalReturnUrl = omniKassaSettings.NormalReturnUrl,
                                AutomaticResponseUrl = omniKassaSettings.AutomaticResponseUrl,
                                InterfaceVersion = omniKassaSettings.InterfaceVersion,
                                OmniKassaUrl = omniKassaSettings.OmniKassaUrl,
                                TestOmniKassaUrl = omniKassaSettings.TestOmniKassaUrl,
                                MinimumAmount = omniKassaSettings.MinimumAmount,
                                AdditionalFee = omniKassaSettings.AdditionalFee,
                                AdditionalFeePercentage = omniKassaSettings.AdditionalFeePercentage,
                                SecretKey = omniKassaSettings.SecretKey,
                                SecretKeyVersion = omniKassaSettings.SecretKeyVersion,
                                TestMode = omniKassaSettings.TestMode,
                                RedirectionTip = omniKassaSettings.RedirectionTip,
                                IsPaymentMethodUserChoice = omniKassaSettings.IsPaymentMethodUserChoice,
                                Ideal = omniKassaSettings.IdealDefaultDisplayName,
                                MasterCard = omniKassaSettings.MasterCardDefaultDisplayName,
                                Visa = omniKassaSettings.VisaDefaultDisplayName,
                                Maestro = omniKassaSettings.MaestroDefaultDisplayName,
                                Acceptgiro = omniKassaSettings.AcceptgiroDefaultDisplayName,
                                MiniTix = omniKassaSettings.MiniTixDefaultDisplayName,
                                Incasso = omniKassaSettings.IncassoDefaultDisplayName,
                                Rembours = omniKassaSettings.RemboursDefaultDisplayName
                            };
         
            Currency primaryStoreCurrency = _currencyService.GetCurrencyById(_currencySettings.PrimaryStoreCurrencyId);
            // Load all available OmniKassa Payments
            model.AvailablePaymentMeanBrandList = new PaymentMeanBrandListModel();
            model.AvailablePaymentMeanBrandList.PaymentMeanBrands = new List<PaymentMeanBrandModel>();

            List<OmniKassaService> oks =
                omniKassaSettings.OmniKassaServicesList.XmlDeserializeFromString<List<OmniKassaService>>();

            foreach (var o in oks)
            {
                model.AvailablePaymentMeanBrandList.PaymentMeanBrands.Add(new PaymentMeanBrandModel(o.Name) {Active = o.Active, AdditionalFee = o.AdditionalFee, AdditionalFeePercentage = o.AdditionalFeePercentage});
            }

            if (primaryStoreCurrency.CurrencyCode != "EUR")
            {
                model.AvailablePaymentMeanBrandList.PaymentMeanBrands.RemoveAll(pb => pb.Name == Constants.Ideal || pb.Name == Constants.MiniTix);
            }
  
            AddLocales(model.Locales, 
                    (locale, languageId) =>
                    {
                        locale.RedirectionTip = _localizationService.GetResource("Plugins.Payments.OmniKassa.RedirectionTip.Localized", languageId, false, "", true);
                        locale.Ideal = _localizationService.GetResource("Plugins.Payments.OmniKassa.IdealDisplayName.Localized", languageId, false, "", true);
                        locale.MasterCard = _localizationService.GetResource("Plugins.Payments.OmniKassa.MasterCardDisplayName.Localized", languageId, false, "", true);
                        locale.Visa = _localizationService.GetResource("Plugins.Payments.OmniKassa.VisaDisplayName.Localized", languageId, false, "", true);
                        locale.Maestro = _localizationService.GetResource("Plugins.Payments.OmniKassa.MaestroDisplayName.Localized", languageId, false, "", true);
                        locale.MiniTix = _localizationService.GetResource("Plugins.Payments.OmniKassa.MiniTixDisplayName.Localized", languageId, false, "", true);
                        locale.Incasso = _localizationService.GetResource("Plugins.Payments.OmniKassa.IncassoDisplayName.Localized", languageId, false, "", true);
                        locale.Rembours = _localizationService.GetResource("Plugins.Payments.OmniKassa.RemboursDisplayName.Localized", languageId, false, "", true);
                        locale.Acceptgiro = _localizationService.GetResource("Plugins.Payments.OmniKassa.AcceptgiroDisplayName.Localized", languageId, false, "", true);
                    });

            return model;
        }

        [NonAction]
        private void AddLocales<TLocalizedModelLocal>( IList<TLocalizedModelLocal> locales, Action<TLocalizedModelLocal, int> configure) where TLocalizedModelLocal : ILocalizedModelLocal
        {
            foreach (var language in _languageService.GetAllLanguages(true))
            {
                var locale = Activator.CreateInstance<TLocalizedModelLocal>();
                locale.LanguageId = language.Id;
                if (configure != null)
                {
                    configure.Invoke(locale, locale.LanguageId);
                }
                locales.Add(locale);
            }
        }

        [NonAction]
        private static void SaveLocalizedResource( ILocalizationService localizationService, int languageId, string localizedText, string resourceName)
        {
            if (localizationService == null)
                throw new ArgumentNullException("localizationService");

            if (languageId == 0)
                throw new ArgumentOutOfRangeException("languageId", "Language ID should not be 0");           
            
            var resource = localizationService.GetLocaleStringResourceByName(resourceName, languageId, false);

            if (resource != null)
            {
                if (string.IsNullOrWhiteSpace(localizedText))
                {
                    //delete
                    localizationService.DeleteLocaleStringResource(resource);
                }
                else
                {
                    //update
                    resource.ResourceValue = localizedText;
                    localizationService.UpdateLocaleStringResource(resource);
                }
            }
            else
            {
                if (!string.IsNullOrWhiteSpace(localizedText))
                {
                    //insert
                    resource = new LocaleStringResource()
                    {
                        LanguageId = languageId,
                        ResourceName = resourceName,
                        ResourceValue = localizedText,
                    };
                    localizationService.InsertLocaleStringResource(resource);
                }
            }
        }

        [ChildActionOnly]
        public ActionResult PaymentInfo()
        {
            var model = new PaymentInfoModel();
            var configmodel = PrepareConfigurationModel(_storeContext.CurrentStore.Id);

            model.IsPaymentMethodUserChoice = configmodel.IsPaymentMethodUserChoice;

            string defaultRedirectionTip = configmodel.RedirectionTip;
            string defaultIdeal = configmodel.Ideal;
            string defaultMasterCard = configmodel.MasterCard;
            string defaultVisa = configmodel.Visa;
            string defaultMaestro = configmodel.Maestro;
            string defaultIncasso = configmodel.Incasso;
            string defaultRembours = configmodel.Rembours;
            string defaultMiniTix = configmodel.MiniTix;
            string defaultAcceptgiro = configmodel.Acceptgiro;

            model.RedirectionTip =
                _localizationService.GetResource("Plugins.Payments.OmniKassa.RedirectionTip.Localized",
                                                 _workContext.WorkingLanguage.Id, false, defaultRedirectionTip, true);


            List<string> activePaymentMeanBrands =
                configmodel.AvailablePaymentMeanBrandList.PaymentMeanBrands.Where(p => p.Active).Select(x => x.Name).ToList();


            if (activePaymentMeanBrands.Contains(Constants.Ideal))
            {
                model.AvailablePaymentsMethods.Add(Constants.Ideal,
                                                   _localizationService.GetResource(
                                                       "Plugins.Payments.OmniKassa.IdealDisplayName.Localized",
                                                       _workContext.WorkingLanguage.Id, false, defaultIdeal, true));
            }

            if (activePaymentMeanBrands.Contains(Constants.MasterCard))
            {
                model.AvailablePaymentsMethods.Add(Constants.MasterCard,
                                                   _localizationService.GetResource(
                                                       "Plugins.Payments.OmniKassa.MasterCardDisplayName.Localized",
                                                       _workContext.WorkingLanguage.Id, false, defaultMasterCard, true));
            }

            if (activePaymentMeanBrands.Contains(Constants.Visa))
            {
                model.AvailablePaymentsMethods.Add(Constants.Visa,
                                                   _localizationService.GetResource(
                                                       "Plugins.Payments.OmniKassa.VisaDisplayName.Localized",
                                                       _workContext.WorkingLanguage.Id, false, defaultVisa, true));
            }

            if (activePaymentMeanBrands.Contains(Constants.Maestro))
            {
                model.AvailablePaymentsMethods.Add(Constants.Maestro,
                                                   _localizationService.GetResource(
                                                       "Plugins.Payments.OmniKassa.MaestroDisplayName.Localized",
                                                       _workContext.WorkingLanguage.Id, false, defaultMaestro, true));
            }

            if (activePaymentMeanBrands.Contains(Constants.Rembours))
            {
                model.AvailablePaymentsMethods.Add(Constants.Rembours,
                                                   _localizationService.GetResource(
                                                       "Plugins.Payments.OmniKassa.RemboursDisplayName.Localized",
                                                       _workContext.WorkingLanguage.Id, false, defaultRembours, true));
            }

            if (activePaymentMeanBrands.Contains(Constants.Incasso))
            {
                model.AvailablePaymentsMethods.Add(Constants.Incasso,
                                                   _localizationService.GetResource(
                                                       "Plugins.Payments.OmniKassa.IncassoDisplayName.Localized",
                                                       _workContext.WorkingLanguage.Id, false, defaultIncasso, true));
            }

            if (activePaymentMeanBrands.Contains(Constants.MiniTix))
            {
                model.AvailablePaymentsMethods.Add(Constants.MiniTix,
                                                   _localizationService.GetResource(
                                                       "Plugins.Payments.OmniKassa.MiniTixDisplayName.Localized",
                                                       _workContext.WorkingLanguage.Id, false, defaultMiniTix, true));
            }

            if (activePaymentMeanBrands.Contains(Constants.Acceptgiro))
            {
                model.AvailablePaymentsMethods.Add(Constants.Acceptgiro,
                                                   _localizationService.GetResource(
                                                       "Plugins.Payments.OmniKassa.AcceptgiroDisplayName.Localized",
                                                       _workContext.WorkingLanguage.Id, false, defaultAcceptgiro, true));
            }

            return View("Nop.Plugin.Payments.OmniKassa.Views.PaymentOmniKassa.PaymentInfo", model);
        }

        /// <summary>
        /// This method is called in the background by the OmniKassa service with the transaction id of the associated payment.
        /// The only task of this method is to validate the payment, no action or redirection is necessary.
        /// </summary>
        /// <returns>Returns EmptyResult</returns>
        [ValidateInput(false)]
        [HttpPost]
        public EmptyResult OmniKassaReport()
        {
            ProcessResponse("OmniKassa Automatic Response");
            return new EmptyResult();
        }

        /// <summary>
        /// This method is called by the OmniKassa service to redirect the user back to the store after the payment. 
        /// </summary>
        /// <returns>Redirects to user to checkout completed or payment failed page.</returns>
        [ValidateInput(false)]
        [HttpPost]
        public ActionResult OmniKassaReturn()
        {
            Order order = ProcessResponse("OmniKassa Manual Response");

            if (order == null)
            {
                return View("Nop.Plugin.Payments.OmniKassa.Views.PaymentOmniKassa.OmniKassaFailed", null);
            }
            
            // If payment is succeded
            if (order.PaymentStatus == PaymentStatus.Paid)
            {
                return RedirectToRoute("CheckoutCompleted", new {orderId = order.Id});
            }
            else
            {
                var model = new PaymentFailedModel()
                                {
                                    OrderId = order.Id,
                                };

                return View("Nop.Plugin.Payments.OmniKassa.Views.PaymentOmniKassa.OmniKassaFailed", model);
            }
        }

        [NonAction]
        public override IList<string> ValidatePaymentForm(FormCollection form)
        {            
            var warnings = new List<string>();

            //validate
            var validator = new PaymentInfoValidator(_localizationService);

            var model = new PaymentInfoModel()
                            {
                                SelectedPaymentMethod = form["SelectedPaymentMethod"],
                                IsPaymentMethodUserChoice = bool.Parse(form["IsPaymentMethodUserChoice"])
                            };

            var validationResult = validator.Validate(model);
            if (!validationResult.IsValid)
                foreach (var error in validationResult.Errors)
                    warnings.Add(error.ErrorMessage);

            return warnings;
        }

        [NonAction]
        public override ProcessPaymentRequest GetPaymentInfo(FormCollection form)
        {
            //I have to use the session 
            // see my issue posted: https://nopcommerce.codeplex.com/workitem/11381
            // and also my forum topic: http://www.nopcommerce.com/boards/t/22676/how-to-get-paymentinfo-in-postprocesspaymentrequest-for-a-redirection-payment-plugin.aspx
            if (bool.Parse(form["IsPaymentMethodUserChoice"]))
            {
                _httpContext.Session["OmniKassa.SelectedPaymentMethod"] = form["SelectedPaymentMethod"];
            }
            var paymentInfo = new ProcessPaymentRequest();
            return paymentInfo;
        }

        #region private methods

        private Order ProcessResponse(string responsesource)
        {
            var storeScope = _storeContext.CurrentStore.Id;
            var omniKassaSettings = _settingService.LoadSetting<OmniKassaPaymentSettings>(storeScope);

            string errorStr;
            String data = Request.Form["Data"];
            String seal = Request.Form["Seal"];

            var processor =
                _paymentService.LoadPaymentMethodBySystemName(PaymentSystemMethodName) as OmniKassaPaymentProcessor;

            if (processor == null ||
                !processor.IsPaymentMethodActive(_paymentSettings) || !processor.PluginDescriptor.Installed)
                throw new NopException("OmniKassa plugin cannot be loaded");

            if (String.IsNullOrEmpty(data))
            {
                errorStr = string.Format("{0} - OmniKassa data field in request not found", responsesource);
                _logger.Error(errorStr);
                return null;
            }

            if (String.IsNullOrEmpty(seal))
            {
                errorStr = string.Format("{0} - OmniKassa seal field in request not found", responsesource);
                _logger.Error(errorStr);
                return null;
            }

            string secretKey = omniKassaSettings.SecretKey;

            // Only for Test Mode 
            if (omniKassaSettings.TestMode)
            {
                secretKey = "002020000000001_KEY1";
            }

            // Check the Seal!!
            string ownSeal = Helpers.EncodeMessage(data, secretKey);

            if (seal.ToLower() != ownSeal.ToLower())
            {
                errorStr = string.Format("{0} - OmniKassa response seal does not match", responsesource);
                _logger.Error(errorStr);
                return null;
            }

            List<KeyValuePair<String, String>> dataItems = ParseData(data);

            String transactionRef = dataItems.First(i => i.Key == "transactionReference").Value;
            if (transactionRef == null)
            {
                errorStr = string.Format("{0} - OmniKassa transactionReference in Data field not found", responsesource);
                _logger.Error(errorStr);
                return null;
            }

            Order order = _orderService.GetOrderByAuthorizationTransactionIdAndPaymentMethod(transactionRef,
                                                                                             PaymentSystemMethodName);

            if (order == null)
            {
                errorStr = string.Format("{0} - OmniKassa Order with transactionRef {1} not found", responsesource,
                                         transactionRef);
                _logger.Error(errorStr);
                return null;
            }

            String responseCode = dataItems.First(i => i.Key == "responseCode").Value;

            if (String.IsNullOrEmpty(responseCode))
            {
                errorStr = string.Format("{0} - OmniKassa responseCode in Data field not found", responsesource);
                _logger.Error(errorStr);
                return null;
            }

            if (responseCode == "00")
            {
                if (_orderProcessingService.CanMarkOrderAsPaid(order))
                {
                    order.AuthorizationTransactionResult = responseCode;
                    _orderService.UpdateOrder(order);

                    _orderProcessingService.MarkOrderAsPaid(order);
                }
            }
            else
            {
                order.AuthorizationTransactionResult = responseCode;
                _orderService.UpdateOrder(order);

                errorStr = string.Format("{0} - Payment for order {1} failed!, Data: {2}", responsesource, order.Id,
                                         Request["data"]);
                _logger.Error(errorStr);
            }

            return order;
        }

        List<KeyValuePair<String, String>> ParseData(String data)
        {
            return (from part in data.Split('|')
                    let key = part.Split('=')[0]
                    let value = part.Split('=')[1]
                    select new KeyValuePair<String, String>(key, value)).ToList();
        }
        
        #endregion
    }
}