﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Security.Cryptography;
using System.Text;
using System.Xml.Serialization;

namespace Nop.Plugin.Payments.OmniKassa
{
    internal static class Helpers
    {
        internal static string EncodeMessage(String rabo, String secretKey)
        {
            Byte[] encodedBytes = new UTF8Encoding().GetBytes(rabo + secretKey);
            byte[] shaResult = new SHA256Managed().ComputeHash(encodedBytes);

            return ByteArrayToHEX(shaResult);
        }

        internal static string ByteArrayToHEX(byte[] bytes)
        {
            StringBuilder result = new StringBuilder(bytes.Length * 2);
            const String hexAlphabet = "0123456789ABCDEF";

            foreach (byte b in bytes)
            {
                result.Append(hexAlphabet[b >> 4]);
                result.Append(hexAlphabet[b & 0xF]);
            }

            return result.ToString();
        }

        // If the primary store currency is not EUR then we have to remove IDEAL and MINITIX because 
        // these payments methods are only supported with EUR currency
        internal static void RemovePaymentsBrandIfNotEur(List<string> list, string currencyCode)
        {
            if (currencyCode == "EUR")
                return;

            list.Remove(Constants.Ideal);
            list.Remove(Constants.MiniTix);
        }

        internal static string XmlSerializeToString<T>(this T toSerialize)
        {
            XmlSerializer xmlSerializer = new XmlSerializer(toSerialize.GetType());
            StringWriter textWriter = new StringWriter();
            xmlSerializer.Serialize(textWriter, toSerialize);
            return textWriter.ToString();
        }

        internal static T XmlDeserializeFromString<T>(this string objectData)
        {
            XmlSerializer xmlSerializer = new XmlSerializer(typeof(T));
            object result;

            using (StringReader reader = new StringReader(objectData))
            {
                result = xmlSerializer.Deserialize(reader);
            }

            return (T) result;
        }
    }
}
