﻿using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Web.Mvc;
using FluentValidation.Attributes;
using Nop.Plugin.Payments.OmniKassa.Validators;
using Nop.Web.Framework;
using Nop.Web.Framework.Localization;
using Nop.Web.Framework.Mvc;

namespace Nop.Plugin.Payments.OmniKassa.Models
{
    [Validator(typeof(ConfigurationValidator))]
    public class ConfigurationModel : BaseNopModel, ILocalizedModel<ConfigurationLocalizedModel>
    {
        public ConfigurationModel()
        {
            AvailablePaymentMeanBrandList = new PaymentMeanBrandListModel();
            AvailablePaymentMeanBrandList.PaymentMeanBrands = new List<PaymentMeanBrandModel>();
            Locales = new List<ConfigurationLocalizedModel>();
        }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.URL")]        
        public string OmniKassaUrl { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.TestURL")] 
        public string TestOmniKassaUrl { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.MerchantID")] 
        public string MerchantId { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.ReturnURL")] 
        public string NormalReturnUrl { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.AutomaticResponseURL")] 
        public string AutomaticResponseUrl { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.InterfaceVersion")]        
        public string InterfaceVersion { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.AdditionalFee")]      
        public decimal AdditionalFee { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.AdditionalFeePercentage")]
        public bool AdditionalFeePercentage { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.MinimumAmount")]  
        public decimal MinimumAmount { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.SecretKey")]  
        public string SecretKey { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.SecretKeyVersion")]
        public string SecretKeyVersion { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.PaymentMeanBrandList")]
       //[UIHint("Nop.Plugin.Payments.OmniKassa.Views.EditorTemplates.PaymentMeanBrandModel")]
        public PaymentMeanBrandListModel AvailablePaymentMeanBrandList { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.TestMode")]  
        public bool TestMode { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.RedirectionTipLabel")]
        [AllowHtml]
        public string RedirectionTip { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.PaymentMethodUserChoice")]
        public bool IsPaymentMethodUserChoice { get; set; }

        public IList<ConfigurationLocalizedModel> Locales { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.IdealLabel")]
        public string Ideal { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.VisaLabel")]
        public string Visa { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.MasterCardLabel")]
        public string MasterCard { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.MaestroLabel")]
        public string Maestro { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.MiniTixLabel")]
        public string MiniTix { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.IncassoLabel")]
        public string Incasso { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.RemboursLabel")]
        public string Rembours { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.AcceptgiroLabel")]
        public string Acceptgiro { get; set; }
    }

    public class ConfigurationLocalizedModel: ILocalizedModelLocal
    {
        public int LanguageId { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.RedirectionTipLabel")]
        [AllowHtml]
        public string RedirectionTip { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.IdealLabel")]
        public string Ideal { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.VisaLabel")]
        public string Visa { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.MasterCardLabel")]
        public string MasterCard { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.MaestroLabel")]
        public string Maestro { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.MiniTixLabel")]
        public string MiniTix { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.IncassoLabel")]
        public string Incasso { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.RemboursLabel")]
        public string Rembours { get; set; }

        [NopResourceDisplayName("Plugins.Payments.OmniKassa.Admin.AcceptgiroLabel")]
        public string Acceptgiro { get; set; }
    }
}