﻿using System.Collections.Generic;
using System.Linq;
using FluentValidation;
using Nop.Plugin.Payments.OmniKassa.Models;
using Nop.Services.Localization;

namespace Nop.Plugin.Payments.OmniKassa.Validators
{
    public class ConfigurationValidator : AbstractValidator<ConfigurationModel>
    {
        public ConfigurationValidator(ILocalizationService localizationService)
        {
            RuleFor(config => config.InterfaceVersion).NotNull().WithMessage(localizationService.GetResource("Plugins.Payments.OmniKassa.Admin.InterfaceVersion.Required"));
            RuleFor(config => config.AvailablePaymentMeanBrandList.PaymentMeanBrands)
                .Must(MustContainActive)
                .WithMessage(localizationService.GetResource("Plugins.Payments.OmniKassa.Admin.AvailablePaymentMeanBrandList"));
             //   .SetValidator(new PaymentMeanBrandListValidator(localizationService));
            
            // Apply the following rules when !NOT! in testmode
            When(config => !config.TestMode, () =>
                                                 {
                                                     RuleFor(config => config.OmniKassaUrl).NotNull().WithMessage(localizationService.GetResource("Plugins.Payments.OmniKassa.Admin.URL.Required"));
                                                     RuleFor(config => config.SecretKey).NotNull().WithMessage(localizationService.GetResource("Plugins.Payments.OmniKassa.Admin.SecretKey.Required"));
                                                     RuleFor(config => config.SecretKeyVersion).NotNull().WithMessage(localizationService.GetResource("Plugins.Payments.OmniKassa.Admin.SecretKeyVersion.Required"));
                                                     RuleFor(config => config.MerchantId).NotNull().WithMessage(localizationService.GetResource("Plugins.Payments.OmniKassa.Admin.MerchantID.Required"));
                                                     RuleFor(config => config.NormalReturnUrl).NotNull().WithMessage(localizationService.GetResource("Plugins.Payments.OmniKassa.Admin.ReturnURL.Required"));
                                                     RuleFor(config => config.AutomaticResponseUrl).NotNull().WithMessage(localizationService.GetResource("Plugins.Payments.OmniKassa.Admin.AutomaticResponseURL.Required"));
                                                 });
            // Test url is required when in testmode
            When(config => config.TestMode, () => RuleFor(config => config.TestOmniKassaUrl).NotNull().WithMessage(localizationService.GetResource("Plugins.Payments.OmniKassa.Admin.TestURL.Required")));            
        }


        private bool MustContainActive(List<PaymentMeanBrandModel> pmb)
        {
            int a = pmb.Count(x => x.Active);

            return a > 0;
        }

    
    }

}